@@ -3,6 +3,7 @@
 import pandas as pd
 import numpy as np
 from sklearn.model_selection import train_test_split
+import logging
 
 import tensorflow as tf
 from tensorflow import keras
@@ -17,10 +18,11 @@ class PublicData:
         :param dataframe: Pandas DataFrame.
         :param continuous_features: List of names of continuous features. The remaining features are categorical features.
         :param outcome_name: Outcome feature name.
-        :param permitted_range (optional): Dictionary with feature names as keys and permitted range as values. Defaults to the range inferred from training data.
+        :param permitted_range (optional): Dictionary with feature names as keys and permitted range in list as values. Defaults to the range inferred from training data.
         :param test_size (optional): Proportion of test set split. Defaults to 0.2.
         :param test_split_random_state (optional): Random state for train test split. Defaults to 17.
         :param continuous_features_precision (optional): Dictionary with feature names as keys and precisions as values.
+        :param data_name (optional): Dataset name
 
         """
 
@@ -54,6 +56,9 @@ class PublicData:
 
         if 'test_size' in params:
             self.test_size = params['test_size']
+            if self.test_size > 1 or self.test_size < 0:
+                raise ValueError(
+                    "should provide a decimal between 0 and 1")
         else:
             self.test_size = 0.2
 
@@ -68,8 +73,11 @@ class PublicData:
             self.continuous_features_precision = None
 
         if len(self.categorical_feature_names) > 0:
+            for feature in self.categorical_feature_names:
+                self.data_df[feature] = self.data_df[feature].apply(str)
             self.data_df[self.categorical_feature_names] = self.data_df[self.categorical_feature_names].astype(
                 'category')
+
         if len(self.continuous_feature_names) > 0:
             for feature in self.continuous_feature_names:
                 if self.get_data_type(feature) == 'float':
@@ -92,9 +100,29 @@ class PublicData:
 
         if 'permitted_range' in params:
             self.permitted_range = params['permitted_range']
+            if not self.check_features_range():
+                raise ValueError(
+                    "permitted range of features should be within their original range")
         else:
             self.permitted_range = self.get_features_range()
 
+        if 'data_name' in params:
+            self.data_name = params['data_name']
+        else:
+            self.data_name = 'mydata'
+
+    def check_features_range(self):
+        for feature in self.continuous_feature_names:
+            if feature in self.permitted_range:
+                min_value = self.train_df[feature].min()
+                max_value = self.train_df[feature].max()
+
+                if self.permitted_range[feature][0] < min_value and self.permitted_range[feature][1] > max_value:
+                    return False
+            else:
+                self.permitted_range[feature] = [self.train_df[feature].min(), self.train_df[feature].max()]
+        return True
+
     def get_features_range(self):
         ranges = {}
         for feature_name in self.continuous_feature_names:
@@ -109,7 +137,7 @@ class PublicData:
         elif((self.data_df[col].dtype == np.float64) or (self.data_df[col].dtype == np.float32)):
             return 'float'
         else:
-            raise ValueError("unknown data type of feature %s" %col)
+            raise ValueError("Unknown data type of feature %s: must be int or float" %col)
 
     def one_hot_encode_data(self, data):
         """One-hot-encodes the data."""
@@ -174,6 +202,17 @@ class PublicData:
                     abs(normalized_train_df[feature].values - np.median(normalized_train_df[feature].values)))
         return mads
 
+    def get_valid_mads(self, normalized=False, display_warnings=False, return_mads=True):
+        """Computes Median Absolute Deviation of features. If they are <=0, returns a practical value instead"""
+        mads = self.get_mads(normalized=normalized)
+        for feature in mads:
+            if mads[feature] <= 0:
+                mads[feature] = 1.0
+                if display_warnings:
+                    logging.warning(" MAD for feature %s is 0, so replacing it with 1.0 to avoid error.", feature)
+        if return_mads:
+            return mads
+
     def get_quantiles_from_training_data(self, quantile=0.05, normalized=False):
         """Computes required quantile of Absolute Deviations of features."""
 
@@ -204,7 +243,8 @@ class PublicData:
         cols = []
         for col_parent in self.categorical_feature_names:
             temp = [self.encoded_feature_names.index(
-                col) for col in self.encoded_feature_names if col.startswith(col_parent)]
+                col) for col in self.encoded_feature_names if col.startswith(col_parent) and
+                   col not in self.continuous_feature_names]
             cols.append(temp)
         return cols
 
@@ -213,15 +253,30 @@ class PublicData:
         if features_to_vary == "all":
             return [i for i in range(len(self.encoded_feature_names))]
         else:
-            return [colidx for colidx, col in enumerate(self.encoded_feature_names) if col.startswith(tuple(features_to_vary))]
+            ixs = []
+            encoded_cats_ixs = self.get_encoded_categorical_feature_indexes()
+            encoded_cats_ixs = [item for sublist in encoded_cats_ixs for item in sublist]
+            for colidx, col in enumerate(self.encoded_feature_names):
+                if colidx in encoded_cats_ixs and col.startswith(tuple(features_to_vary)):
+                    ixs.append(colidx)
+                elif colidx not in encoded_cats_ixs and col in features_to_vary:
+                    ixs.append(colidx)
+            return ixs
 
     def from_dummies(self, data, prefix_sep='_'):
         """Gets the original data from dummy encoded data with k levels."""
         out = data.copy()
-        for l in self.categorical_feature_names:
+        for feat in self.categorical_feature_names:
+            # first, derive column names in the one-hot-encoded data from the original data
+            cat_col_values = []
+            for val in list(self.data_df[feat].unique()):
+                cat_col_values.append(feat + prefix_sep + str(val)) # join original feature name and its unique values , ex: education_school
+            match_cols = [c for c in data.columns if c in cat_col_values] # check for the above matching columns in the encoded data
+
+            # then, recreate original data by removing the suffixes - based on the GitHub issue comment: https://github.com/pandas-dev/pandas/issues/8745#issuecomment-417861271
             cols, labs = [[c.replace(
-                x, "") for c in data.columns if l+prefix_sep in c] for x in ["", l+prefix_sep]]
-            out[l] = pd.Categorical(
+                x, "") for c in match_cols] for x in ["", feat + prefix_sep]]
+            out[feat] = pd.Categorical(
                 np.array(labs)[np.argmax(data[cols].values, axis=1)])
             out.drop(cols, axis=1, inplace=True)
         return out
@@ -240,7 +295,6 @@ class PublicData:
                     prec = len(str(modes[mx]).split('.')[1])
                     if prec > maxp:
                         maxp = prec
-                maxp = min(maxp, 4)
                 precisions[ix] = maxp
         return precisions
 
